@@ -61,7 +61,7 @@
 #include "sudo_auth.h"
 
 #ifndef lint
-static const char rcsid[] = "$Sudo: passwd.c,v 1.10 2002/01/17 05:24:28 millert Exp $";
+static const char rcsid[] = "$Sudo: passwd.c,v 1.11 2002/01/17 15:56:15 millert Exp $";
 #endif /* lint */
 
 #define DESLEN			13
@@ -75,7 +75,7 @@ passwd_init(pw, promptp, auth)
 {
 #ifdef HAVE_SKEYACCESS
     if (skeyaccess(pw, user_tty, NULL, NULL) == 0)
-	return(AUTH_FATAL);
+	return(AUTH_FAILURE);
 #endif
     return(AUTH_SUCCESS);
 }
