@@ -62,7 +62,7 @@
 #include "sudo.h"
 
 #ifndef lint
-static const char rcsid[] = "$Sudo: env.c,v 1.13 2001/12/31 05:53:23 millert Exp $";
+static const char rcsid[] = "$Sudo: env.c,v 1.16 2002/04/18 15:38:52 millert Exp $";
 #endif /* lint */
 
 /*
@@ -157,29 +157,32 @@ zero_env(envp)
 	    case 'H':
 		if (strncmp("HOME=", *ep, 5) == 0)
 		    break;
+		continue;
 	    case 'L':
 		if (strncmp("LOGNAME=", *ep, 8) == 0)
 		    break;
+		continue;
 	    case 'P':
 		if (strncmp("PATH=", *ep, 5) == 0) {
 		    user_path = *ep + 5;
 		    /* XXX - set to sane default instead of user's? */
 		    break;
 		}
+		continue;
 	    case 'S':
-		if (strncmp("SHELL=", *ep, 6) == 0) {
+		if (strncmp("SHELL=", *ep, 6) == 0)
 		    user_shell = *ep + 6;
-		    continue;
-		} else if (!user_prompt && !strncmp("SUDO_PROMPT=", *ep, 12)) {
+		else if (!user_prompt && !strncmp("SUDO_PROMPT=", *ep, 12))
 		    user_prompt = *ep + 12;
-		    continue;
-		}
+		continue;
 	    case 'T':
 		if (strncmp("TZ=", *ep, 3) == 0)
 		    break;
+		continue;
 	    case 'U':
 		if (strncmp("USER=", *ep, 5) == 0)
 		    break;
+		continue;
 	    default:
 		continue;
 	}
@@ -338,7 +341,7 @@ rebuild_env(sudo_mode, envp)
 	if (!(didvar & DID_HOME))
 	    *nep++ = format_env("HOME", user_dir);
 	if (!(didvar & DID_SHELL))
-	    *nep++ = format_env("SHELL", user_shell);
+	    *nep++ = format_env("SHELL", sudo_user.pw->pw_shell);
 	if (!(didvar & DID_LOGNAME))
 	    *nep++ = format_env("LOGNAME", user_name);
 	if (!(didvar & DID_USER))
@@ -431,19 +434,16 @@ rebuild_env(sudo_mode, envp)
 
     /* Add the SUDO_COMMAND envariable (cmnd + args). */
     if (user_args) {
-	cp = emalloc(strlen(user_cmnd) + strlen(user_args) + 15);
-	sprintf(cp, "SUDO_COMMAND=%s %s", user_cmnd, user_args);
+	easprintf(&cp, "SUDO_COMMAND=%s %s", user_cmnd, user_args);
 	insert_env(newenvp, cp);
     } else
 	insert_env(newenvp, format_env("SUDO_COMMAND", user_cmnd));
 
     /* Add the SUDO_USER, SUDO_UID, SUDO_GID environment variables. */
     insert_env(newenvp, format_env("SUDO_USER", user_name));
-    cp = emalloc(MAX_UID_T_LEN + 10);
-    sprintf(cp, "SUDO_UID=%ld", (long) user_uid);
+    easprintf(&cp, "SUDO_UID=%ld", (long) user_uid);
     insert_env(newenvp, cp);
-    cp = emalloc(MAX_UID_T_LEN + 10);
-    sprintf(cp, "SUDO_GID=%ld", (long) user_gid);
+    easprintf(&cp, "SUDO_GID=%ld", (long) user_gid);
     insert_env(newenvp, cp);
 
     return(newenvp);
